using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using Outlook = Microsoft.Office.Interop.Outlook;
using Office = Microsoft.Office.Core;

namespace OlCustomMenuBar
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("D8D1F9BB-A3BA-4D70-AFF7-2DB2BEDB418B"), ProgId("OlCustomMenuBar.AddinModule"), RunInstaller(true)]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            InitializeComponent();
        }
 
        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;
 
        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
			// 
			// AddinModule
			// 
			this.AddinName = "OlCustomMenuBar";
			this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
			this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);
			this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);

		}
        #endregion
 
        #region Add-in Express automatic code
 
        // Required by Add-in Express - do not modify
        // the methods within this region
 
        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

		private AddinExpress.MSO.ADXCommandBar menuBar = null;

		private void AddMenuControls()
		{
			AddinExpress.MSO.ADXCommandBarControl ctrl;

			ctrl = AddControl(menuBar, typeof(AddinExpress.MSO.ADXCommandBarPopup), 
				"My Popup Menu", "MY_UNIQUE_TAG 1", 0, false, null, true);
			AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton), 
				"Popup Item 1", "MY_UNIQUE_TAG 2", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
			AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton), 
				"Popup Item 2", "MY_UNIQUE_TAG 3", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
			AddControl(ctrl, typeof(AddinExpress.MSO.ADXCommandBarButton), 
				"Popup Item 3", "MY_UNIQUE_TAG 4", 0, false, new AddinExpress.MSO.ADXClick_EventHandler(DoClick), true);
		}

		private void AddinModule_AddinStartupComplete(object sender, System.EventArgs e)
		{
			Outlook.Explorer expl = null;

			try
			{
				menuBar = new AddinExpress.MSO.ADXCommandBar();
				menuBar.Position = AddinExpress.MSO.ADXMsoBarPosition.adxMsoBarTop;
				menuBar.Protection = AddinExpress.MSO.ADXMsoBarProtection.adxMsoBarNoMove;

				expl = OutlookApp.ActiveExplorer();
				this.CommandBarsObj = expl.CommandBars;
				menuBar.CommandBarObj = (this.CommandBarsObj as Office.CommandBars).Add(
					"MyCustomMenuBar", Office.MsoBarPosition.msoBarTop, true, true) as Office.CommandBar;
				if (menuBar.IsConnected)
				{
					this.AddToContainer(menuBar);
					menuBar.UpdateCommandBarObj(this);
					AddMenuControls();
				}
			}
			finally
			{
				if (expl != null)
					Marshal.ReleaseComObject(expl);
				this.CommandBarsObj = null;
			}
		}

		private void AddinModule_AddinBeginShutdown(object sender, System.EventArgs e)
		{
			//
		}

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

		private AddinExpress.MSO.ADXCommandBarControl AddControl(object container, Type ctrlType, string caption, 
			string controlTag, int before, bool beginGroup, object eventHandler, bool temporary)
		{
			AddinExpress.MSO.ADXCommandBarControl adxNewControl = null;
			AddinExpress.MSO.ADXBaseControlCollection collection = null;
			
			if (container is AddinExpress.MSO.ADXCommandBar)
				collection = (container as AddinExpress.MSO.ADXCommandBar).Controls;
			else if (container is AddinExpress.MSO.ADXCommandBarPopup)
				collection = (container as AddinExpress.MSO.ADXCommandBarPopup).Controls;
			
			if (collection == null) return null;

			if (eventHandler != null)
			{
				adxNewControl = collection.GetControlByControlTag(controlTag, false);
				if (adxNewControl != null)
				{
					switch(ctrlType.Name)
					{
						case "ADXCommandBarButton":
							adxNewControl.AsButton.Click -= (eventHandler as AddinExpress.MSO.ADXClick_EventHandler);
							break;
						case "ADXCommandBarComboBox":
							adxNewControl.AsComboBox.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
						case "ADXCommandBarDropDownList":
							adxNewControl.AsDropDownList.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
						case "ADXCommandBarEdit":
							adxNewControl.AsEdit.Change -= (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
							break;
					}
				}
			}

			adxNewControl = collection.Add(ctrlType, controlTag, 1, before, temporary);
			adxNewControl.BeginGroup = beginGroup;
			adxNewControl.Caption = caption;
			switch(ctrlType.Name)
			{
				case "ADXCommandBarButton":
					adxNewControl.AsButton.Style = AddinExpress.MSO.ADXMsoButtonStyle.adxMsoButtonCaption;
					if (eventHandler != null)
						adxNewControl.AsButton.Click += (eventHandler as AddinExpress.MSO.ADXClick_EventHandler);
					break;
				case "ADXCommandBarComboBox":
					if (eventHandler != null)
						adxNewControl.AsComboBox.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarDropDownList":
					if (eventHandler != null)
						adxNewControl.AsDropDownList.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarEdit":
					if (eventHandler != null)
						adxNewControl.AsEdit.Change += (eventHandler as AddinExpress.MSO.ADXChange_EventHandler);
					break;
				case "ADXCommandBarPopup":
					break;
			}
			return adxNewControl;
		}

		private void DoClick(object sender)
		{
			MessageBox.Show((sender as AddinExpress.MSO.ADXCommandBarControl).Caption, AddinName);
		}
	}
}

